package net.hoopajoo.cheeselab3;
import java.io.*;

import java.util.zip.*;

public class CheeseTx {
	int sequence, channel, length;
	String raw_line, msgtype, enc;
	byte[] buffer;
	
	public static int send_tx( PrintStream out, int last_sendid, int channel, 
			String msgtype, String data ) { 
		return( send_tx( out, last_sendid, channel, msgtype, data.getBytes() ) );
	}
	
	public static int send_tx( PrintStream out, int last_sendid, int channel, 
			String msgtype, byte[] data ) {
		try {
			CheeseLab.dmesg( 3, "Sending transaction: " + msgtype );
			last_sendid++;
			//out.print( last_sendid + " " + channel + " " + 
			//		msgtype + " binary " + data.length + "\n" );
		
			byte[] newdata = data;
			String encoding = "binary";
			
			if( data.length > 60 ) {
				CheeseLab.dmesg( 3, "Large packet, compressing" );
				newdata = gzip( data );
				encoding = "gzip";
			}
			
			out.print( last_sendid + " " + channel + " " + 
				msgtype + " " + encoding + " " + newdata.length + "\n" );
				
			out.write( newdata );
			out.flush();
		}catch( Exception e ) {
			e.printStackTrace();
			return( -1 );
		}
		
		if( out.checkError() ) {
			return( -1 );
		}
		
		return( last_sendid );
	}
	
	public static CheeseTx accept_tx( BufferedInputStream in ) {
		CheeseTx ret = new CheeseTx();

		CheeseLab.dmesg( 2, "Accepting tx" );
		try {
			String lin = readline( in );

			if( lin == null ) {
				CheeseLab.dmesg( 2, "Got null tx line" );
				return( null );
			}
			
			ret.raw_line = lin;
			
			String[] comp = lin.split( "\\s+" );
			ret.sequence = Integer.parseInt( comp[ 0 ] );
			ret.channel = Integer.parseInt( comp[ 1 ] );
			ret.msgtype = comp[ 2 ];
			ret.enc = comp[ 3 ];
			ret.length = Integer.parseInt( comp[ 4 ] );
	
			CheeseLab.dmesg( 3, "Msgtype is " + ret.msgtype +
				" of length " + ret.length );
	
			ret.buffer = new byte[ ret.length ];
			int total_read = 0;
			while( total_read < ret.length ) {
				int left_to_read = ret.length - total_read;
				CheeseLab.dmesg( 4, "Need to read: " + left_to_read );
				int read = in.read( ret.buffer, total_read, left_to_read );
				CheeseLab.dmesg( 4, "Read: " + read );
				
				if( read == -1 ) {
					return( null );
				}
				total_read += read;
			}
			
			if( ret.enc.equals( "gzip" ) ) {
				ret.buffer = gzip_dash_d( ret.buffer );
			}
			
			//lin = new String( buffer );
			//dmesg( "Got data:\n======\n" + lin + "\n======" );
		}catch( Exception e ) {
			e.printStackTrace();
			return( null );
		}
		
		CheeseLab.dmesg( 3, "Accept tx complete" );
		return( ret );
	}
	
	public static String readline( BufferedInputStream s ) {
		String r = new String();
		byte[] bytes = new byte[ 1 ];
		boolean looping = true;
		
		try {
			while( looping ) {
				int bb = s.read();
				if( bb == -1 ) {
					return( null );
				}
				bytes[ 0 ] = (byte)bb;
				r += new String( bytes );
				if( bytes[ 0 ] == '\n' )
					looping = false;
			}
		}catch( Exception e ) {
			e.printStackTrace();
			return( null );
		}
		
		return( r );
	}
	
	public static byte[] gzip_dash_d( byte[] b ) {
		// decompress, assume no more than 10:1 compression
		byte[] output_buffer = new byte[ 0 ],
			inbytes = new byte[ 1 ];
		
		try {
			GZIPInputStream in = new GZIPInputStream( new ByteArrayInputStream( b ) );
			ByteArrayOutputStream out = new ByteArrayOutputStream();
			while( in.read( inbytes, 0, 1 ) != -1 ) {
				out.write( inbytes );
			}
			out.close();
			in.close();
			output_buffer = out.toByteArray();
		}catch( Exception e ) {
			e.printStackTrace();
		}
		
		CheeseLab.dmesg( 3, "Decompression input size: " + b.length + " output: " + output_buffer.length );
		return( output_buffer );
	}
	
	public static byte[] gzip( byte[] b ) {
		// Assume we will not do more than double the size :)
		byte[] output_buffer = new byte[ 0 ];
		
		try {
			ByteArrayOutputStream output = new ByteArrayOutputStream();
			GZIPOutputStream out = new GZIPOutputStream( output );
			
			out.write( b );
			out.close();
			
			output_buffer = output.toByteArray();
		}catch( Exception e ) {
			e.printStackTrace();
		}
		
		CheeseLab.dmesg( 3, "Compression input size: " + b.length + " output: " + output_buffer.length );
		return( output_buffer );
	}
}
